import { __awaiter } from 'tslib';
import { SchematicsException, chain } from '@angular-devkit/schematics';
import { insertImport, addImportToModule } from '@schematics/angular/utility/ast-utils';
import { createSourceFile, ScriptTarget } from 'typescript';
import { normalize, workspaces } from '@angular-devkit/core';

const packageName = '@rx-angular/template';

/**
 * An operation that does nothing.
 */
class NoopChange {
    constructor() {
        this.description = 'No operation.';
        this.order = Infinity;
        this.path = null;
    }
    apply() {
        return Promise.resolve();
    }
}
/**
 * Will add text to the source code.
 */
class InsertChange {
    constructor(path, pos, toAdd) {
        this.path = path;
        this.pos = pos;
        this.toAdd = toAdd;
        if (pos < 0) {
            throw new Error('Negative positions are invalid');
        }
        this.description = `Inserted ${toAdd} into position ${pos} of ${path}`;
        this.order = pos;
    }
    /**
     * This method does not insert spaces if there is none in the original string.
     */
    apply(host) {
        return host.read(this.path).then((content) => {
            const prefix = content.substring(0, this.pos);
            const suffix = content.substring(this.pos);
            return host.write(this.path, `${prefix}${this.toAdd}${suffix}`);
        });
    }
}

/**
 * @license
 * Copyright Google Inc. All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://angular.io/license
 */
const MODULE_EXT = '.module.ts';
const ROUTING_MODULE_EXT = '-routing.module.ts';
/**
 * Find the module referred by a set of options passed to the schematics.
 */
function findRootModule(host, module, rootPath = '', skipImport = false) {
    if (skipImport || !module) {
        return undefined;
    }
    const modulePath = normalize(`${rootPath}/${module}`);
    if (host.exists(modulePath)) {
        return modulePath;
    }
    else if (host.exists(modulePath + '.ts')) {
        return normalize(modulePath + '.ts');
    }
    else if (host.exists(modulePath + MODULE_EXT)) {
        return normalize(modulePath + MODULE_EXT);
    }
    else if (host.exists(`${modulePath}/${module}${MODULE_EXT}`)) {
        return normalize(`${modulePath}/${module}${MODULE_EXT}`);
    }
    else if (host.exists(`${modulePath}/${module}.ts`)) {
        return normalize(`${modulePath}/${module}.ts`);
    }
    else {
        throw new Error(`Specified module path ${modulePath} does not exist`);
    }
}

function getWorkspace(tree, path = '/') {
    return __awaiter(this, void 0, void 0, function* () {
        const host = createHost(tree);
        const { workspace } = yield workspaces.readWorkspace(path, host);
        return workspace;
    });
}
function createHost(tree) {
    return {
        readFile(path) {
            return __awaiter(this, void 0, void 0, function* () {
                const data = tree.read(path);
                if (!data) {
                    throw new Error('File not found.');
                }
                return data.toString();
            });
        },
        writeFile(path, data) {
            return __awaiter(this, void 0, void 0, function* () {
                return tree.overwrite(path, data);
            });
        },
        isDirectory(path) {
            return __awaiter(this, void 0, void 0, function* () {
                // approximate a directory check
                return !tree.exists(path) && tree.getDir(path).subfiles.length > 0;
            });
        },
        isFile(path) {
            return __awaiter(this, void 0, void 0, function* () {
                return tree.exists(path);
            });
        },
    };
}

function getProject(host, projectName) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const workspace = yield getWorkspace(host);
        if (workspace.projects.has(projectName)) {
            return workspace.projects.get(projectName);
        }
        else if (((_a = workspace.extensions) === null || _a === void 0 ? void 0 : _a.defaultProject) &&
            workspace.projects.has(workspace.extensions.defaultProject)) {
            return workspace.projects.get(workspace.extensions.defaultProject);
        }
        throw new SchematicsException('Could not find project "' + projectName + '"');
    });
}

function getModuleFile(tree, options) {
    const modulePath = options.module;
    if (!tree.exists(modulePath)) {
        throw new SchematicsException(`File ${modulePath} does not exist.`);
    }
    const text = tree.read(modulePath);
    if (text === null) {
        throw new SchematicsException(`File ${modulePath} does not exist.`);
    }
    const sourceText = text.toString('utf-8');
    return createSourceFile(modulePath, sourceText, ScriptTarget.Latest, true);
}
function applyChanges(tree, path, changes) {
    const recorder = tree.beginUpdate(path);
    for (const change of changes) {
        recorder.insertLeft(change.pos, change.toAdd);
    }
    tree.commitUpdate(recorder);
    return tree;
}
function addImportsToModuleFile(options, imports, importName = packageName) {
    return (tree) => {
        const module = getModuleFile(tree, options);
        const importChanges = insertImport(module, options.module, imports.join(', '), importName);
        return applyChanges(tree, options.module, [
            importChanges,
        ]);
    };
}
function addImportsToModuleDeclaration(options, imports) {
    return (tree) => {
        const module = getModuleFile(tree, options);
        const importChanges = imports.map((imp) => addImportToModule(module, options.module, imp, packageName)[0]);
        return applyChanges(tree, options.module, importChanges);
    };
}
function ngAdd(options) {
    return (tree) => __awaiter(this, void 0, void 0, function* () {
        var _a;
        const project = yield getProject(tree, options.project);
        const sourceRoot = (_a = (project && project.sourceRoot)) !== null && _a !== void 0 ? _a : 'src';
        const modulesToAdd = ['LetModule', 'PushModule'];
        options.module = findRootModule(tree, options.module, sourceRoot);
        return chain([
            addImportsToModuleFile(options, modulesToAdd),
            addImportsToModuleDeclaration(options, modulesToAdd),
        ]);
    });
}

/**
 * Generated bundle index. Do not edit.
 */

export { ngAdd };
//# sourceMappingURL=rx-angular-template-schematics.js.map
